@@ -1,9 +1,10 @@
-from __future__ import absolute_import
+# -*- coding: utf-8 -*-
+from __future__ import absolute_import, unicode_literals
 
 import os
+import sys
 
 import pytest
-
 from envyaml import EnvYAML
 
 # set os env
@@ -30,7 +31,7 @@ def test_it_should_read_env_file():
     assert env["PASSWORD_WE"] == "env-password-without-escape"
     assert env["EMPTY"] == ""
 
-    # Test wrong names, should be not found and raise KeyErro
+    # Test wrong names, should be not found and raise KeyError
     with pytest.raises(KeyError):
         assert env["01sre"]
 
@@ -290,4 +291,55 @@ def test_it_should_parser_environment_inside_array_and_object():
     env = EnvYAML("tests/env.test.yaml", env_file="tests/test.env")
 
     # assert array
-    assert env['var_in_array.to.0'] == 'env-username'
+    assert env["var_in_array.to.0"] == "env-username"
+
+    # assert dictionary
+    assert env["var_in_dict.extra.user"] == "env-username"
+    assert env["var_in_dict.extra.password"] == "env-password-with-escape"
+
+
+def test_it_should_parser_long_env_with_several_elements():
+    env = EnvYAML("tests/env.test.yaml", env_file="tests/test.env")
+
+    assert env["key_extr"] == 'project-x -ex "es5" -an -c:v libx264 -qp 23 -f seg'
+
+
+def test_it_should_has_no_stict_exception_when_set_env():
+    # set special env to supress strict mode globaly
+    os.environ[EnvYAML.ENVYAML_STRICT_DISABLE] = ""
+
+    env = EnvYAML("tests/env.ignored.yaml", strict=True)
+
+    assert env["env_file.config"] == "$ENV_CONFIG_VERSION"
+
+    del os.environ[EnvYAML.ENVYAML_STRICT_DISABLE]
+
+
+def test_it_should_parse_env_file_as_list():
+    env = EnvYAML("tests/env.list.yaml", env_file="tests/test.env")
+
+    assert env["0.testing_1.env.username"] == "env-username"
+    assert env["1.testing_2.env.username"] == "env-username"
+    assert env["2.testing_3.env.username"] == "env-username"
+
+
+@pytest.mark.skipif(sys.version_info.major == 2, reason='Ignore UTF8 at Python 2.7')
+def test_it_should_parse_env_file_as_unicode():
+    va = "ÜBERMORGEN"
+    vb = "ПІСЛЯЗАВТРА"
+
+    os.environ[va] = va + "😃"
+    os.environ[vb] = vb + "😃"
+
+    env = EnvYAML("tests/env.default.yaml", "tests/test.env")
+
+    assert env["next.relase"] == va + "😃"
+    assert env["next.maybe"] == vb + "😃"
+
+    del os.environ[va]
+    del os.environ[vb]
+
+
+def test_it_should_thwor_exception_when_double_variable_in_dotenv_file():
+    with pytest.raises(ValueError):
+        EnvYAML("tests/env.default.yaml", "tests/double.env")
